"""
This file contains a class that uses 3 asterisks to create a very commonly found loading animation found on Arch Linux. Again, you can pass a function as an argument, and pass the function's arguments using args and kwargs. You can run the file, a small sample function has been used to demonstrate the effect of the animation.
"""

import threading
import time
import sys

from SampleProgram import sample_usage

class AsteriskProcessing:

    def __init__(self, function, args=(), kwargs={}):
        """
        The constructor accepts any function. Its arguments can be passed using args and kwargs. It sets all of these things as attrubutes. It creates a thread to run the passed function concurrently, and loads the animation on the STDOUT.
        """
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.done = False
        self.thread = threading.Thread(target=self.run_function)
        self.thread.start()
        self.animation()

    def run_function(self):
        """
        Calls the passed function with the passed arguments.
        """
        self.function(*self.args, **self.kwargs)
        self.done = True

    def animation(self):
        """
        The patty of the burger. Does the animation on the screen, as long as the function is executing.
        """
        string_list = [
            "  *****  ",
            " *****   ",
            "*****    ",
            " *****   ",
            "  *****  ",
            "   ***** ",
            "    *****",
        ]
        idx = 0
        while not self.done:
            sys.stdout.write("\r" + string_list[idx])
            sys.stdout.flush()
            idx = (idx + 1) % len(string_list)
            time.sleep(0.2)

        sys.stdout.write("\rDone        \n")

if __name__ == "__main__":
    asterisk_processing = AsteriskProcessing(sample_usage, args=("John Doe", 24), kwargs=({"occupation": "Software Engineer", "location": "World"}))



"""
This file contains a class that creates a very simple loading screen on the terminal window. You can pass a function
as an argument to the constructor, and its arguments using args, or keyword args using kwargs. You can run the file to understand its effect.
"""

import threading
import time
import sys

from SampleProgram import sample_usage 

class SimpleLoadingScreen:
    def __init__(self, function, args=(), kwargs={}):
        """
        The constructor accepts the function, and its args and kwargs. Sets them as attributes,
        Starts a thread to run the function concurrently, and loads the animation.
        """
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.done = False
        self.thread = threading.Thread(target=self.run_function)
        self.thread.start()
        self.animation()

    def run_function(self):
        """
        Calls the passed function with the passed arguments.
        """
        self.function(*self.args, **self.kwargs)
        self.done = True

    def animation(self):
        """
        The goey of the marshmello. Does the animation on the screen, as long as the function is executing.
        """
        loading_chars = "|/-\\"
        idx = 0
        while not self.done:
            sys.stdout.write("\r" + loading_chars[idx])
            sys.stdout.flush()
            idx = (idx + 1) % len(loading_chars)
            time.sleep(0.1)

        sys.stdout.write("\rDone          \n")
        

if __name__ == "__main__":
    loading_screen = SimpleLoadingScreen(sample_usage, args=("John Doe", 24), kwargs={"occupation": "Software Engineer","location": "World"})




import time

def sample_usage(name, age, occupation=None, location=None):
    """
    A sample function to demonstrate the usage of the above created animation.
    """
    print(f"Hello, {name}! You are {age} years old.")
    if occupation:
        print(f"You work as a {occupation}.")
    if location:
        print(f"You live in {location}.")
    time.sleep(5)



from setuptools import setup

setup(
    name="pythonspinners",
    version="0.1.0a",
    author="Abhiram Mangipudi",
    author_email="k0mplexontuta@tutamail.com",
    description="A simple package that provided loading screen objects for a passed function.",
    packages=["pythonspinners"],
)
